@@ -1,11 +1,12 @@
 /*
  * Javassist, a Java-bytecode translator toolkit.
- * Copyright (C) 1999-2007 Shigeru Chiba. All Rights Reserved.
+ * Copyright (C) 1999- Shigeru Chiba. All Rights Reserved.
  *
  * The contents of this file are subject to the Mozilla Public License Version
  * 1.1 (the "License"); you may not use this file except in compliance with
  * the License.  Alternatively, the contents of this file may be used under
- * the terms of the GNU Lesser General Public License Version 2.1 or later.
+ * the terms of the GNU Lesser General Public License Version 2.1 or later,
+ * or the Apache License Version 2.0.
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -19,7 +20,6 @@ import java.io.DataInputStream;
 import java.io.DataOutputStream;
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
 import java.util.Map;
@@ -40,17 +40,35 @@ public final class ClassFile {
     int[] interfaces;
     ArrayList fields;
     ArrayList methods;
-    LinkedList attributes;
+    ArrayList attributes;
     String thisclassname; // not JVM-internal name
     String[] cachedInterfaces;
     String cachedSuperclass;
 
+    /**
+     * The major version number of class files
+     * for JDK 1.1.
+     */
+    public static final int JAVA_1 = 45;
+
+    /**
+     * The major version number of class files
+     * for JDK 1.2.
+     */
+    public static final int JAVA_2 = 46;
+
     /**
      * The major version number of class files
      * for JDK 1.3.
      */
     public static final int JAVA_3 = 47;
 
+    /**
+     * The major version number of class files
+     * for JDK 1.4.
+     */
+    public static final int JAVA_4 = 48;
+
     /**
      * The major version number of class files
      * for JDK 1.5.
@@ -63,10 +81,19 @@ public final class ClassFile {
      */
     public static final int JAVA_6 = 50;
 
+    /**
+     * The major version number of class files
+     * for JDK 1.7.
+     */
+    public static final int JAVA_7 = 51;
+
     /**
      * The major version number of class files created
-     * from scratch.  The default value is 47 (JDK 1.3)
-     * or 49 (JDK 1.5) if the JVM supports <code>java.lang.StringBuilder</code>.
+     * from scratch.  The default value is 47 (JDK 1.3).
+     * It is 49 (JDK 1.5)
+     * if the JVM supports <code>java.lang.StringBuilder</code>.
+     * It is 50 (JDK 1.6)
+     * if the JVM supports <code>java.util.zip.DeflaterInputStream</code>.
      */
     public static int MAJOR_VERSION = JAVA_3;
 
@@ -74,6 +101,8 @@ public final class ClassFile {
         try {
             Class.forName("java.lang.StringBuilder");
             MAJOR_VERSION = JAVA_5;
+            Class.forName("java.util.zip.DeflaterInputStream");
+            MAJOR_VERSION = JAVA_6;
         }
         catch (Throwable t) {}
     }
@@ -111,7 +140,7 @@ public final class ClassFile {
         methods = new ArrayList();
         thisclassname = classname;
 
-        attributes = new LinkedList();
+        attributes = new ArrayList();
         attributes.add(new SourceFileAttribute(constPool,
                 getSourcefileName(thisclassname)));
     }
@@ -185,7 +214,7 @@ public final class ClassFile {
      */
     public void prune() {
         ConstPool cp = compact0();
-        LinkedList newAttributes = new LinkedList();
+        ArrayList newAttributes = new ArrayList();
         AttributeInfo invisibleAnnotations
             = getAttribute(AnnotationsAttribute.invisibleTag);
         if (invisibleAnnotations != null) {
@@ -222,7 +251,6 @@ public final class ClassFile {
         }
 
         attributes = newAttributes;
-        cp.prune();
         constPool = cp;
     }
 
@@ -386,12 +414,14 @@ public final class ClassFile {
         newname = Descriptor.toJvmName(newname);
         constPool.renameClass(oldname, newname);
 
+        AttributeInfo.renameClass(attributes, oldname, newname);
         list = methods;
         n = list.size();
         for (int i = 0; i < n; ++i) {
             MethodInfo minfo = (MethodInfo)list.get(i);
             String desc = minfo.getDescriptor();
             minfo.setDescriptor(Descriptor.rename(desc, oldname, newname));
+            AttributeInfo.renameClass(minfo.getAttributes(), oldname, newname);
         }
 
         list = fields;
@@ -400,6 +430,7 @@ public final class ClassFile {
             FieldInfo finfo = (FieldInfo)list.get(i);
             String desc = finfo.getDescriptor();
             finfo.setDescriptor(Descriptor.rename(desc, oldname, newname));
+            AttributeInfo.renameClass(finfo.getAttributes(), oldname, newname);
         }
     }
 
@@ -420,12 +451,14 @@ public final class ClassFile {
 
         constPool.renameClass(classnames);
 
+        AttributeInfo.renameClass(attributes, classnames);
         ArrayList list = methods;
         int n = list.size();
         for (int i = 0; i < n; ++i) {
             MethodInfo minfo = (MethodInfo)list.get(i);
             String desc = minfo.getDescriptor();
             minfo.setDescriptor(Descriptor.rename(desc, classnames));
+            AttributeInfo.renameClass(minfo.getAttributes(), classnames);
         }
 
         list = fields;
@@ -434,6 +467,34 @@ public final class ClassFile {
             FieldInfo finfo = (FieldInfo)list.get(i);
             String desc = finfo.getDescriptor();
             finfo.setDescriptor(Descriptor.rename(desc, classnames));
+            AttributeInfo.renameClass(finfo.getAttributes(), classnames);
+        }
+    }
+
+    /**
+     * Internal-use only.
+     * <code>CtClass.getRefClasses()</code> calls this method. 
+     */
+    public final void getRefClasses(Map classnames) {
+        constPool.renameClass(classnames);
+
+        AttributeInfo.getRefClasses(attributes, classnames);
+        ArrayList list = methods;
+        int n = list.size();
+        for (int i = 0; i < n; ++i) {
+            MethodInfo minfo = (MethodInfo)list.get(i);
+            String desc = minfo.getDescriptor();
+            Descriptor.rename(desc, classnames);
+            AttributeInfo.getRefClasses(minfo.getAttributes(), classnames);
+        }
+
+        list = fields;
+        n = list.size();
+        for (int i = 0; i < n; ++i) {
+            FieldInfo finfo = (FieldInfo)list.get(i);
+            String desc = finfo.getDescriptor();
+            Descriptor.rename(desc, classnames);
+            AttributeInfo.getRefClasses(finfo.getAttributes(), classnames);
         }
     }
 
@@ -516,7 +577,15 @@ public final class ClassFile {
         fields.add(finfo);
     }
 
-    private void addField0(FieldInfo finfo) {
+    /**
+     * Just appends a field to the class.
+     * It does not check field duplication.
+     * Use this method only when minimizing performance overheads
+     * is seriously required.
+     *
+     * @since 3.13
+     */
+    public final void addField2(FieldInfo finfo) {
         fields.add(finfo);
     }
 
@@ -544,7 +613,7 @@ public final class ClassFile {
      * Returns the method with the specified name. If there are multiple methods
      * with that name, this method returns one of them.
      * 
-     * @return null if no such a method is found.
+     * @return null if no such method is found.
      */
     public MethodInfo getMethod(String name) {
         ArrayList list = methods;
@@ -578,7 +647,15 @@ public final class ClassFile {
         methods.add(minfo);
     }
 
-    private void addMethod0(MethodInfo minfo) {
+    /**
+     * Just appends a method to the class.
+     * It does not check method duplication or remove a bridge method.
+     * Use this method only when minimizing performance overheads
+     * is seriously required.
+     *
+     * @since 3.13
+     */
+    public final void addMethod2(MethodInfo minfo) {
         methods.add(minfo);
     }
 
@@ -646,7 +723,7 @@ public final class ClassFile {
      * @see #getAttributes()
      */
     public AttributeInfo getAttribute(String name) {
-        LinkedList list = attributes;
+        ArrayList list = attributes;
         int n = list.size();
         for (int i = 0; i < n; ++i) {
             AttributeInfo ai = (AttributeInfo)list.get(i);
@@ -708,14 +785,14 @@ public final class ClassFile {
         n = in.readUnsignedShort();
         fields = new ArrayList();
         for (i = 0; i < n; ++i)
-            addField0(new FieldInfo(cp, in));
+            addField2(new FieldInfo(cp, in));
 
         n = in.readUnsignedShort();
         methods = new ArrayList();
         for (i = 0; i < n; ++i)
-            addMethod0(new MethodInfo(cp, in));
+            addMethod2(new MethodInfo(cp, in));
 
-        attributes = new LinkedList();
+        attributes = new ArrayList();
         n = in.readUnsignedShort();
         for (i = 0; i < n; ++i)
             addAttribute(AttributeInfo.read(cp, in));
@@ -724,7 +801,7 @@ public final class ClassFile {
     }
 
     /**
-     * Writes a class file represened by this object into an output stream.
+     * Writes a class file represented by this object into an output stream.
      */
     public void write(DataOutputStream out) throws IOException {
         int i, n;
