@@ -206,7 +206,7 @@ public class TranslationString {
 
 		// パフォーマンスを考慮し可能な限り正規表現は使用しない
 
-		if (str.trim().equals("")) {
+		if (str.trim().length() == 0) {
 			starts.append(str);
 			body = "";
 			return;
